/** this is to be contributed to https://github.com/DefinitelyTyped/DefinitelyTyped#how-can-i-contribute */

declare module '@voxeet/voxeet-web-sdk' {
  enum Quality {
    Auto = 'auto',
    HD = 'hd',
    MD = 'md',
    SD = 'sd',
  }

  enum ConferenceStatus {
    Created = 'created',
    Destroyed = 'destroyed',
    Ended = 'ended',
    Error = 'error',
    Joined = 'joined',
    Left = 'left',
  }

  enum ParticipantStatus {
    /* A participant successfully connected to a conference. */
    CONNECTED = 'Connected',
    /* A participant received the conference invitation and is connecting to a conference. */
    CONNECTING = 'Connecting',
    /* A peer connection failed, and the participant cannot connect to a conference. */
    ERROR = 'Error',
    /* A participant did not enable audio, video, or screen-share and, therefore, is not connected to any stream. */
    INACTIVE = 'Inactive',
    /* A participant left the conference. */
    LEFT = 'Left',
    /* A participant is invited to a conference but has not received an invitation yet. */
    RESERVED = 'Reserved',
    /* A participant experiences a peer connection problem, which may result in the Error or Connected status. */
    WARNING = 'Warning',
  }

  enum ParticipantType {
    /* The Participant is a listener. */
    LISTENER = 'listener',
    /* The Participant is a speaker. */
    SPEAKER = 'speaker',
    /* The Participant is a user. */
    USER = 'user',
  }

  enum VideoPresentationState {
    /* Presentation is paused. */
    PAUSED = 'paused',

    /* Presentation is playing. */
    PLAYING = 'playing',

    /* Presentation is stopped. */
    STOPPED = 'stopped',
  }

  enum MediaStreamType {
    ScreenShare = 'ScreenShare',
    Camera = 'Camera',
  }

  export interface MediaStreamWithType extends MediaStream {
    type: MediaStreamType;
  }

  export interface ConferenceParameters {
    // currentLanguage = data.currentLanguage;
    dolbyVoice: boolean;
    liveRecording: boolean;
    // mode = data.mode;
    // multiSession = data.multiSession;
    rtcpMode: 'max' | 'worst' | 'average';
    // stats = data.stats;
    ttl: number;
    videoCodec: 'H264' | 'VP8';
  }

  export interface ConferenceOptions {
    alias?: string;
    params?: ConferenceParameters;
    pinCode?: number;
  }

  export interface MixingOptions {
    enabled: boolean;
  }

  export interface JoinOptions {
    /*
     * Sets the conference WebRTC constraints. By default, only audio is enabled: {audio: true, video: false}.
     */
    constraints?: any;

    /*
     * Joins a conference as Mixer type.
     */
    mixing?: MixingOptions;

    /*
     * Indicates the preference for receiving audio, either stereo or mono.
     */
    preferRecvMono?: boolean;

    /*
     * Indicates the preference for audio transmission, either stereo or mono.
     */
    preferSendMono?: boolean;

    /*
     * Activates the sending side of the simulcast.
     */
    simulcast?: boolean;
  }

  export interface AudioProcessingSenderOptions {
    audioProcessing?: boolean;
  }

  export interface AudioProcessingOptions {
    send?: AudioProcessingSenderOptions;
  }

  export interface ParticipantQuality {
    id: string;
    quality: Quality;
  }

  export interface ParticipantInfo {
    avatarUrl?: string;
    externalId?: string;
    name?: string;
  }

  export interface QualityIndicator {
    audio: number;
    video: number;
  }

  export interface Recording {
    /* The ID of the participant who started the recording. */
    participantId: string;

    /* The recording timestamp. */
    startTimestamp: number;
  }

  export interface FilePresentation {
    /* The file id. */
    id: string;

    /* The number of images in the presentation. */
    imageCount?: number;

    /* The file owner. */
    owner: Participant;

    /* The current position in the presentation. */
    position?: number = 0;
  }

  export interface VideoPresentation {
    /* The participant that started the presentation. */
    owner: Participant;

    /* The current video presentation timestamp used for seeking and pausing the video. */
    timestamp: number;

    /* The URL of the video presentation. */
    url: string;
  }

  export class Conference {
    get alias(): string;
    get id(): string;
    get isNew(): boolean;
    get params(): ConferenceParameters;
    get pinCode(): string;
    get status(): ConferenceStatus;
  }

  export class Participant extends EventEmitter {
    audio: boolean;
    id: string;
    info: ParticipantInfo;
    status: ParticipantStatus;
    streams: MediaStream[];
    type: ParticipantType;
  }

  export class Service {
    on(event: string | symbol, listener: (...args: any[]) => void): this;
    off(event: string | symbol, listener: (...args: any[]) => void): this;
  }

  export class SessionService extends Service {
    /**
     * Opens a new session.
     */
    open(info: ParticipantInfo): Promise<void>;

    /**
     * Closes the current session.
     */
    close(): void;

    /**
     * Provides the local participant object that belongs to the current session.
     */
    get participant(): Participant;
  }

  export class MediaDeviceService extends Service {
    /**
     * Enumerates audio devices.
     */
    enumerateAudioDevices(type: 'input' | 'output'): Promise<MediaDeviceInfo[]>;

    /**
     * Enumerates video devices.
     */
    enumerateVideoDevices(type: 'input' | 'output'): Promise<MediaDeviceInfo[]>;

    /**
     * Enables the selection of an audio device input.
     */
    selectAudioInput(deviceId: string): Promise<MediaDeviceInfo>;

    /**
     * Enables the selection of an audio device output.
     */
    selectAudioOutput(deviceId: string): Promise<MediaDeviceInfo>;

    /**
     * Enables the selection of a video device input.
     */
    selectVideoInput(deviceId: string, constraints?: any): Promise<any>;
  }

  export class ConferenceService extends Service {
    /**
     * Creates a conference with [ConferenceOptions]
     */
    public create(options: ConferenceOptions): Promise<Conference>;
    /**
     * Provides a Conference object that allows joining a conference.
     */
    public fetch(conferenceId: string): Promise<Conference>;

    /**
     * Creates and joins a demo conference.
     */
    public demo(): Promise<Conference>;
    /**
     * Replays a previously recorded conference.
     * See also: [join](#join), [listen](#listen)
     */
    public replay(conference: Conference, offset: number, mixingOptions?: MixingOptions): Promise<Conference>;
    /**
     * Joins a conference as a listener. You can choose to either [join](#join), [replay](#replay), or [listen](#listen) to a conference. The `listen` method connects to the conference in the receiving only mode which does not allow transmitting video or audio.
     * See also: [join](#join), [replay](#replay)
     */
    public listen(conference: Conference): Promise<Conference>;
    /**
     * Joins the conference.
     *
     * **Note**: Due to a known Firefox issue, a user who has never permitted Firefox to use a microphone and camera cannot join a conference as a listener. If you want to join a conference as a listener using the Firefox browser, make sure that Firefox has permission to use your camera and microphone. To check the permissions, follow these steps:
     *
     * **1.** Select the lock icon in the address bar. </br>
     * **2.** Select the right arrow placed next to `Connection Secure`. </br>
     * **3.** Select `More information`. </br>
     * **4.** Go to the `Permissions` tab. </br>
     * **5.** Look for the `Use the camera` and `Use the microphone` permission and select the `Allow` option.
     *
     * See also: [listen](#listen), [replay](#replay)
     *
     *
     *```javascript
     * // For example
     * const constraints = {
     *   audio: true,
     *   video: {
     *     width: {
     *       min: "320",
     *       max: "1280"
     *     },
     *     height: {
     *       min: "240",
     *       max: "720"
     *     }
     *   }
     * };
     *
     * // A simplest example of constraints would be:
     * const constraints = {audio: true, video: true};
     *
     * VoxeetSDK.conference.join(conference, {constraints: constraints})
     *   .then((info) => {
     *   })
     *   .catch((error) => {
     *   });
     * ```
     */
    public join(conference: Conference, options: JoinOptions): Promise<Conference>;

    /**
     * Leaves the conference.
     */
    public leave({ leaveRoom = true, reason = 'hangUp', keepAlive = false } = {}): Promise<void>;
    /**
     * Allows a specific participant to play audio that is blocked by the browser's auto-play policy.
     */
    public playBlockedAudio(): any;

    /**
     * Stops playing the specified remote participants' audio to the local participant or stops playing the local participant's audio to the conference.  The mute method does not notify the server to stop audio stream transmission. To stop sending an audio stream to the server or to stop receiving an audio stream from the server, use the [stopAudio](#stopaudio) method.
     *
     * The mute method depends on the Dolby Voice usage:
     *
     * - In conferences where Dolby Voice is not enabled, conference participants can mute themselves or remote participants.
     * - In conferences where Dolby Voice is enabled, conference participants can only mute themselves.
     *
     * **Note**: In SDK 2.4 and prior releases, if a conference participant calls the mute method, empty frames are sent to the other participants. Due to a Safari issue, participants who join a conference using Safari and start receiving the empty frames can experience a Safari crash. Due to a different API implementation in SDK 3.0, this problem does not occur during Dolby Voice conferences.
     *
     */
    public mute(participant: Participant, isMuted: boolean): any;

    /**
     * Gets the current mute state of the local participant.
     */
    public isMuted(): boolean;

    /**
     * Gets the participant's audio level. The possible values of the audio level are in range from 0.0 to 1.0 point.
     *
     * **Note**: This API is no longer supported for remote participants when the client connects to a Dolby Voice conference.
     *
     */
    public audioLevel(participant: Participant, callback: () => void): any;

    /**
     * Gets the participant's current speaking status for an active talker indicator.
     */
    public isSpeaking(participant: Participant, callback: () => void): boolean;

    /**
     * Notifies the server to either start sending the local participant's video stream to the conference or start sending a remote participant's video stream to the local participant. The startVideo method does not control the remote participant's video stream; if a remote participant does not transmit any video stream, the local participant cannot change it using the startVideo method.
     *
     *```javascript
     * const video = {
     *     mandatory: {
     *       minWidth: 160,
     *       minHeight: 120,
     *       maxWidth: 320,
     *       maxHeight: 240,
     *       minFrameRate: 10,
     *       maxFrameRate: 30,
     *    }
     * };
     *
     * voxeet.conference.startVideo(participantId, constraints)
     *  .then(() => {
     *
     *  });
     * ```
     */
    public startVideo(participant: Participant, constraints?: any): Promise<any>;

    /**
     * Notifies the server to either stop sending the local participant's video stream to the conference or stop sending a remote participant's video stream to the local participant.
     */
    public stopVideo(participant: Participant): Promise<void>;

    /**
     * Enables and disables audio processing for the local participant.
     */
    public audioProcessing(participant: Participant, options: AudioProcessingOptions): Promise<void>;

    /**
     * Notifies the server to either:
     *
     * - Start sending the local participant's audio stream to the conference, or
     *
     * - Start sending a remote participant's audio stream to the local participant
     *
     * If the SDK does not transmit a specific remote participant's audio stream to the local participant, the local participant can enable the transmission of this stream through the startAudio method. In such a case, only the local participant starts receiving the audio stream; the startAudio method does not impact the rest of the conference participants.
     *
     * If a remote participant does not transmit any audio stream, the local participant cannot change it using the startAudio method.
     *
     * **Note**: This API is no longer supported for remote participants when the client connects to a Dolby Voice conference.
     *
     * The Voxeet SDK automatically manages audio rendering and, therefore, the application does not need to implement its own
     * `<audio>` element. The application can use [selectAudioInput](mediadeviceservice#selectaudioinput) and
     * [selectAudioOutput](mediadeviceservice#selectaudiooutput) methods to select the proper audio input and output devices.
     *
     *
     */
    public startAudio(participant: Participant): Promise<any>;

    /**
     * Notifies the server to either:
     *
     * - Stop sending the local participant's audio stream to the conference, or
     *
     * - Stop sending a remote participant's audio stream to the local participant
     *
     * If the SDK transmits a specific remote participant's audio stream to the local participant, the local participant can disable the transmission of this stream through the stopAudio method. In such a case, only the local participant stops receiving the audio stream; the stopAudio method does not impact the rest of the conference participants.
     *
     * If a remote participant transmits an audio stream, the local participant cannot change it using the stopAudio method.
     *
     * **Note**: This API is no longer supported for remote participants when the client connects to a Dolby Voice conference.
     *
     */
    public stopAudio(participant): Promise<any>;

    /**
     * Starts a screen sharing session.
     *
     *```javascript
     * VoxeetSDK.conference.startScreenShare()
     *   .then(() => {
     *
     *   })
     *   .catch(e => {
     *
     *   });
     * ````
     */
    public startScreenShare(sourceId: any): Promise<any>;

    /**
     * Stops the screen-sharing session.
     */
    public stopScreenShare(): Promise<any>;

    /**
     * Provides [standard WebRTC statistics](https://www.w3.org/TR/webrtc-stats/#dom-rtcstatstype) for the application. Based on the WebRTC statistics, the SDK computes [audio and video statistics](/developers/interactivity-apis/client-sdk/reference-javascript/model/qualityindicator).
     */
    public localStats(): Promise<void>;

    /**
     * Configures the quality of the received Simulcast streams.
     *
     */
    public simulcast(requested: ParticipantQuality[]): any;

    /**
     * Sends the notifications to the conference participants to inform them about the invitation to the conference. The use of [externalId](/developers/interactivity-apis/client-sdk/reference-javascript/model/participantinfo#optional-externalid) from the [ParticipantInfo](model/participantinfo) model is mandatory.
     *
     * **Note**: Use the invite method only after joining a conference. For more information about using invitations, see the [Using Invitations](/developers/interactivity-apis/client-sdk/using-invitations) article.
     *
     * ```javascript
     * VoxeetSDK.conference.invite(conference: Conference, externalId: externalId)
     * ```
     *
     */
    public invite(conference: Conference, participants: ParticipantInfo[]): Promise<any>;

    /*
     * The Mean Opinion Score (MOS) which represents the participant's audio and video quality. The SDK calculates the audio and video quality scores, rounds them up, and displays as integer values in a rage from 1 to 5, where 1 represents the worst quality and 5 represents the highest quality. In cases when the MOS score is not available, the SDK returns the value -1.
     *
     * **Note**: With SDK 3.0, audio Mean Opinion Scores (MOS) are unavailable for web clients connected to Dolby Voice conferences.
     */
    public qualityIndicators(stats: Record<string, QualityIndicator>): Promise<void>;

    /**
     * Provides a list of conference participants.
     */
    get participants(): Record<string, Participant>;

    /**
     * Returns the current conference.
     */
    get current(): Conference;
  }

  export class RecordingService extends Service {
    /**
     * Starts recording the current conference.
     */
    start(): any;

    /**
     * Stops recording the current conference.
     */
    stop(): any;

    /**
     * Returns the current recording.
     */
    get current(): Recording;
  }

  export class FilePresentationService extends Service {
    /**
     * Converts a file before file presentation. The file is uploaded as FormData.
     *
     * Supported file formats are:
     * - doc/docx (Microsoft Word)
     * - ppt/pptx
     * - pdf
     *
     * After conversion, the files are broken into individual images with maximum resolution capped at __2560x1600__.
     *
     * When the file is converted and ready for the presentation, the application will receive a [converted](#converted) event.
     */
    convert(file: string);

    /**
     * Starts a file presentation after the file is converted by the voxeet service.
     */
    start(file: FileConverted);

    /**
     * Updates the presentation to the corresponding page.
     */
    update(page: number): Promise<any>;

    /**
     * Stops the presentation
     */
    stop(): Promise<any>;

    /**
     * Returns the URL of a page as image during file presentation.
     */
    image(page: number, fileId?: string): Promise<string>;

    /**
     * Provides the URL of a thumbnail (referring to a specific page of a file) during file presentation.
     */
    thumbnail(page: number, fileId?: string): Promise<string>;

    /**
     * Provides the current file presentation.
     */
    get current(): FilePresentation;
  }

  export class VideoPresentationService extends Service {
    /**
     * Starts a video presentation.
     */
    start(url: string): any;

    /**
     * Stops the current video presentation.
     */
    stop(): any;

    /**
     * Plays the current video presentation.
     */
    play(): any;

    /**
     * Pauses the current video presentation.
     */
    pause(timestamp: number): any;

    /**
     * Seeks the current video presentation.
     */
    seek(timestamp: number): any;

    /**
     * Provides the current video presentation.
     */
    get current(): VideoPresentation | null;

    /**
     * Provides the current state of the video presentation.
     */
    get state(): VideoPresentationState;
  }

  export class CommandService extends Service {
    /**
     * Sends a message (basic string) to all other participants of a specific conference
     */
    send(message: string): Promise<void>;
  }

  export class VoxeetSDK extends EventEmitter {
    version: string;

    /**
     * The Session Service.
     */
    static get session(): SessionService;
    /**
     * Retrieves the unique [MediaDeviceService](/developers/interactivity-apis/client-sdk/reference-javascript/mediadeviceservice) instance,
     * which enables the participant to interact with devices through the system.
     */
    static get mediaDevice(): MediaDeviceService;

    /**
     * Retrieves the unique [ConferenceService](/developers/interactivity-apis/client-sdk/reference-javascript/conferenceservice) instance,
     * which enables the participant to interact with conferences.
     */
    static get conference(): ConferenceService;

    /**
     * Retrieves the unique [RecordingService](/developers/interactivity-apis/client-sdk/reference-javascript/recordingservice) instance,
     * which enables the participant to use the recording option.
     */
    static get recording(): RecordingService;

    /**
     * Retrieves the unique [FilePresentationService](/developers/interactivity-apis/client-sdk/reference-javascript/filepresentationservice) instance,
     * which enables the participant to use file presentations.
     */
    static get filePresentation(): FilePresentationService;

    /**
     * Retrieves the unique [VideoPresentationService](//developers/interactivity-apis/client-sdk/reference-javascript/videopresentationservice) instance,
     * which enables the participant to use video presentations.
     */
    static get videoPresentation(): VideoPresentationService;

    /**
     * Retrieves the unique [CommandService](/developers/interactivity-apis/client-sdk/reference-javascript/commandservice) instance,
     * which enables the participant to send messages in a string shape into specified conferences.
     */
    static get command(): CommandService;

    /**
     * Initializes the SDK using the customer key and secret.
     */
    static initialize(customerKey: string, customerSecret: string);

    /**
     * Initialize the SDK with an access token that is provided by the customer backend communicating with Voxeet servers.
     * It Allows securing the customer key and secret.
     *
     * The authentication flow is as follow:
     *
     * ```
     * Client          Customer Server       Voxeet Server
     *   |                    |                    |
     *   |  Get Access Token  |  /oauth2/token (1) |
     *   |------------------->|------------------->|
     *   |    Access Token    |    Access Token    |
     *   |<-------------------|<-------------------|
     *   |  initializeToken(accessToken, callback) |
     *   |---------------------------------------->|
     * ```
     *
     * The access token has a limited period of validity and needs to be refreshed for security reasons. In such case,
     * The Voxeet SDK will call the callback provided to initializeToken. This one must return a Promise<string>
     * containing the refreshed access token by calling the customer backend.
     *
     * ```
     * Client          Customer Server       Voxeet Server
     *   |      callback      | /oauth2/refresh (2)|
     *   |------------------->|------------------->|
     *   |    Access Token    |    Access Token    |
     *   |<-------------------|<-------------------|
     * ```
     *
     * (1) and (2) are two REST API endpoints available on Voxeet servers and documented on the developer portal.
     *
     */
    static initializeToken(accessToken: string, callback: () => void);
  }
  export default VoxeetSDK;
}
